/*!
 * \file    llwu_common.c
 * \brief   common LLWU routines
 *
 * This file defines the functions/interrupt handlers/macros used for LLWU to be used as wakeup source.
 * And some common initializations.
 *
 * \version $Revision: 1.0 $
 * \author  Philip Drake(rxaa60)
 ***/

#include "common.h"
#include "llwu_common.h"
#include "mcg.h"

extern void fb_clk_init(void);
extern void clkout_init(unsigned char clockout_select, unsigned char clkoutdiv);

extern int core_clk_khz;
extern int core_clk_mhz;

/* function: llwu_configure

   description: Set up the LLWU for wakeup the MCU from LLS and VLLSx modes 
   from the selected pin or module.
   
   inputs:
   pin_en - unsigned integer, bit position indicates the pin is enabled.  
            More than one bit can be set to enable more than one pin at a time.  
   
   rise_fall - 0x00 = External input disabled as wakeup
               0x01 - External input enabled as rising edge detection
               0x02 - External input enabled as falling edge detection
               0x03 - External input enablge as any edge detection
   module_en - unsigned char, bit position indicates the module is enabled.  
               More than one bit can be set to enabled more than one module                   
   
   for example:  if bit 0 and 1 need to be enabled as rising edge detect call this  routine with
   pin_en = 0x0003 and rise_fall = 0x02
   
   Note: to set up one set of pins for rising and another for falling, 2 calls to this 
         function are required, 1st for rising then the second for falling.
   
*/
void llwu_reset_enable(void){
    print_str(" LLWU Reset pin enabled as wakeup source from Low power modes \n");
    LLWU_RST = LLWU_RST_LLRSTE_MASK;   //no reset filter for now 
}
void llwu_configure(unsigned int pin_en, unsigned char rise_fall, unsigned char module_en ) {
    print_str(" LLWU package pins #  64 48 44 32  Pin functions \n");

    if( pin_en & 0x0001){
	     LLWU_PE1 |= LLWU_PE1_WUPE0(rise_fall); 
	     print_str(" LLWU configured pins  4  -  -  -  PTC7/UART0_RX/I2C0_SDA/RGPIO7/SPI1_MISO/FBa_AD12 to be an LLWU wakeup source \n");
       LLWU_F1 |= LLWU_F1_WUF0_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0002){
       LLWU_PE1 |= LLWU_PE1_WUPE1(rise_fall); 
	     print_str(" LLWU configured pins  6  2  -  -  PTD1/UART0_RTS_b/I2C1_SCL/RGPIO9/SPI1_SS/FBa_AD14/I2S0_RX_BCLK to be an LLWU wakeup source \n");
       LLWU_F1 |= LLWU_F1_WUF1_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0004){
       LLWU_PE1 |= LLWU_PE1_WUPE2(rise_fall); 
	     print_str(" LLWU configured pins 12  8  6  6 ADC0_SE3/PTA5/UART1_RTS_b/I2C2_SDA/FTM1_CH5/SPI1_MOSI/CLKOUT/I2S0_TXD to be an LLWU wakeup source \n");
       LLWU_F1 |= LLWU_F1_WUF2_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0008){
       LLWU_PE1 |= LLWU_PE1_WUPE3(rise_fall); 
	     print_str(" LLWU configured pins 30 23 21 16 ADC0_SE13/TSIO_CH5/PTA7/UART0_TX/FTM0_QD_PHA/FBa_D5 to be an LLWU wakeup source \n");
       LLWU_F1 |= LLWU_F1_WUF3_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0010){
       LLWU_PE2 |= LLWU_PE2_WUPE4(rise_fall); 
	     print_str(" LLWU configured pins 32  -  -  - ADC0_SE15/TSI0_CH7/PTD7/UART0_CTS_b/I2C3_SCL/RGPIO15/FBa_D3 to be an LLWU wakeup source \n");
       LLWU_F1 |= LLWU_F1_WUF4_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0020){
       LLWU_PE2 |= LLWU_PE2_WUPE5(rise_fall); 
	     print_str(" LLWU configured pins 35 25 23 17 IRQ/PTB0/I2C0_SCL/IRQ/EZP_CS_b to be an LLWU wakeup source \n");
       LLWU_F1 |= LLWU_F1_WUF5_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0040){
       LLWU_PE2 |= LLWU_PE2_WUPE6(rise_fall); 
	     print_str(" LLWU configured pins 36 26 24 18 TSI0_CH10/PTB1/SPI0_SCLK/I2C0_SDA/FTM_FLT2/LPTMR_ALT2/FTM0_QD_PHB/FB_CLKOUT to be an LLWU wakeup source \n");
       LLWU_F1 |= LLWU_F1_WUF6_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0080){
       LLWU_PE2 |= LLWU_PE2_WUPE7(rise_fall); 
	     print_str(" LLWU configured pins 39 27 25 19 ADC0_SE17/TSI0_CH13/PTB2/SPI0_MISO/FBa_CS0_b to be an LLWU wakeup source \n");
       LLWU_F1 |= LLWU_F1_WUF7_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0100){
       LLWU_PE3 |= LLWU_PE3_WUPE8(rise_fall); 
	     print_str(" LLWU configured pins 44 31 27  - ADC0_SE22/PTE7/UART0_TX/PDB0_EXTRG/SPI1_MOSI/FBa_RW_b/FBa_AD4 to be an LLWU wakeup source \n");
       LLWU_F2 |= LLWU_F2_WUF8_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0200){
       LLWU_PE3 |= LLWU_PE3_WUPE9(rise_fall); 
	     print_str(" LLWU configured pins 45 32 28 21 BKGD/MS/PTB4 to be an LLWU wakeup source \n");
       LLWU_F2 |= LLWU_F2_WUF9_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0400){
       LLWU_PE3 |= LLWU_PE3_WUPE10(rise_fall); 
	     print_str(" LLWU configured pins 55  -  -  - CMP0_IN1/PTF2/SPI0_MISO/FBa_AD7 to be an LLWU wakeup source \n");
       LLWU_F2 |= LLWU_F2_WUF10_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x0800){
       LLWU_PE3 |= LLWU_PE3_WUPE11(rise_fall); 
	     print_str(" LLWU configured pins 56 40 36  - CMP0_IN2/PTF3/SPI0_MOSI/RGPIO1/FBa_AD8/I2S0_TXD to be an LLWU wakeup source \n");
       LLWU_F2 |= LLWU_F2_WUF11_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x1000){
       LLWU_PE4 |= LLWU_PE4_WUPE12(rise_fall); 
	     print_str(" LLWU configured pins 57 41 37 29 CMP0_IN3/PTC2/UART1_RTS_b/SPI1_SS/RGPIO2/FBa_AD18/I2S0_TX_FS to be an LLWU wakeup source \n");
       LLWU_F2 |= LLWU_F2_WUF12_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x2000){
       LLWU_PE4 |= LLWU_PE4_WUPE13(rise_fall); 
	     print_str(" LLWU configured pins 59 43 39  - PTF5/UART1_RX/SPI1_MISO/FBa_D2/FBa_RW_b/I2S0_RXD to be an LLWU wakeup source \n");
       LLWU_F2 |= LLWU_F2_WUF13_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x4000){
       LLWU_PE4 |= LLWU_PE4_WUPE14(rise_fall); 
	     print_str(" LLWU configured pins 62 46 42 30 PTC3/UART0_CTS_b RGPIO3 SPI0_SCLK CLKOUT USB_CLKIN/I2S0_MCLK/2S0_CLKIN to be an LLWU wakeup source \n");
       LLWU_F2 |= LLWU_F2_WUF14_MASK;   // write one to clear the flag
    }
    if( pin_en & 0x8000){
       LLWU_PE4 |= LLWU_PE4_WUPE15(rise_fall); 
	     print_str(" LLWU configured pins 63 47 43 31 PTC4/UART0_RX/RGPIO4/SPI0_MISO/PDB0_EXTRG/USB_SOF_PULSE to be an LLWU wakeup source \n");
       LLWU_F2 |= LLWU_F2_WUF15_MASK;   // write one to clear the flag
    }
    LLWU_ME = module_en;  //Set up modules to wakeup up
    
    print_str("LLWU PE1 = %#02X \r\n",   (LLWU_PE1)) ;
    print_str("LLWU PE2 = %#02X \r\n",   (LLWU_PE2));
    print_str("LLWU PE3 = %#02X \r\n",   (LLWU_PE3)) ;
    print_str("LLWU PE4 = %#02X \r\n",   (LLWU_PE4));
    print_str("LLWU ME = %#02X \r\n",    (LLWU_ME)) ;
    print_str("LLWU F1 = %#02X \r\n",    (LLWU_F1)) ;
    print_str("LLWU F2 = %#02X \r\n",    (LLWU_F2)) ;
    print_str("LLWU F3 = %#02X \r\n",    (LLWU_F3)) ; 
    print_str("LLWU FILT1 = %#02X \r\n", (LLWU_FILT1))  ;
    print_str("LLWU FILT2 = %#02X \r\n", (LLWU_FILT2))  ;
    print_str("LLWU RST   = %#02X \r\n", (LLWU_RST))  ;
	  //function ends
}

//Interrupt handler for LLWU 
interrupt void llwu_isr(void){
	//Disable Watchdog Timer
	SIM_COPC = SIM_COPC_COPT(0);

	//Release hold if not already done so.
	if (PMC_REGSC & PMC_REGSC_ACKISO_MASK)
			  PMC_REGSC |= PMC_REGSC_ACKISO_MASK;

//	print_str("\n[LLWU_ISR] ****LLWU ISR entered*****\r\n");
   if (LLWU_F1 & LLWU_F1_WUF0_MASK) {
       LLWU_F1 |= LLWU_F1_WUF0_MASK;   // write one to clear the flag
//       print_str("\n[LLWU ISR]  ****WUF0 was set *****\r\n"); 
   }
   if (LLWU_F1 & LLWU_F1_WUF1_MASK) {
//       print_str("\n [LLWU ISR] ****WUF1 was set *****\r\n"); 
       LLWU_F1 |= LLWU_F1_WUF2_MASK;   // write one to clear the flag
   }
   if (LLWU_F1 & LLWU_F1_WUF2_MASK) {
//       print_str("\n [LLWU ISR] ****WUF2 was set *****\r\n"); 
       LLWU_F1 |= LLWU_F1_WUF2_MASK;   // write one to clear the flag
   }
   if (LLWU_F1 & LLWU_F1_WUF3_MASK){
//       print_str("\n [LLWU ISR] ****WUF3 was set *****\r\n");
       LLWU_F1 |= LLWU_F1_WUF3_MASK;   // write one to clear the flag
    }
   if (LLWU_F1 & LLWU_F1_WUF4_MASK) {
 //      print_str("\n [LLWU ISR] ****WUF4 was set *****\r\n"); 
       LLWU_F1 |= LLWU_F1_WUF4_MASK;   // write one to clear the flag
   }
   if (LLWU_F1 & LLWU_F1_WUF5_MASK) {
//       print_str("\n [LLWU ISR] ****WUF5 was set *****\r\n"); 
       LLWU_F1 |= LLWU_F1_WUF5_MASK;   // write one to clear the flag
   }
   if (LLWU_F1 & LLWU_F1_WUF6_MASK) {
//       print_str("\n [LLWU ISR] ****WUF6 was set *****\r\n"); 
       LLWU_F1 |= LLWU_F1_WUF6_MASK;   // write one to clear the flag
    }
   if (LLWU_F1 & LLWU_F1_WUF7_MASK) {
       print_str("\n [LLWU ISR] ****WUF7 was set *****\r\n"); 
       LLWU_F1 |= LLWU_F1_WUF7_MASK;   // write one to clear the flag
   }
   if (LLWU_F2 & LLWU_F2_WUF8_MASK) {
 //      print_str("\n [LLWU ISR] ****WUF8 was set *****\r\n"); 
       LLWU_F2 |= LLWU_F2_WUF8_MASK;   // write one to clear the flag
   }
   if (LLWU_F2 & LLWU_F2_WUF9_MASK) {
//       print_str("\n [LLWU ISR] ****WUF9 was set *****\r\n"); 
       LLWU_F2 |= LLWU_F2_WUF9_MASK;   // write one to clear the flag
   }
   if (LLWU_F2 & LLWU_F2_WUF10_MASK) {
//       print_str("\n [LLWU ISR] ****WUF10 was set *****\r\n"); 
       LLWU_F2 |= LLWU_F2_WUF10_MASK;   // write one to clear the flag
   }
   if (LLWU_F2 & LLWU_F2_WUF11_MASK) {
 //      print_str("\n [LLWU ISR] ****WUF11 was set *****\r\n"); 
       LLWU_F2 |= LLWU_F2_WUF11_MASK;   // write one to clear the flag
   }
   if (LLWU_F2 & LLWU_F2_WUF12_MASK) {
//       print_str("\n [LLWU ISR] ****WUF12 was set *****\r\n"); 
       LLWU_F2 |= LLWU_F2_WUF12_MASK;   // write one to clear the flag
   }
   if (LLWU_F2 & LLWU_F2_WUF13_MASK) {
//       print_str("\n [LLWU ISR] ****WUF13 was set *****\r\n"); 
       LLWU_F2 |= LLWU_F2_WUF13_MASK;   // write one to clear the flag
   }
   if (LLWU_F2 & LLWU_F2_WUF14_MASK) {
//       print_str("\n [LLWU ISR] ****WUF14 was set *****\r\n"); 
       LLWU_F2 |= LLWU_F2_WUF14_MASK;   // write one to clear the flag
   }
   if (LLWU_F2 & LLWU_F2_WUF15_MASK) {
 //      print_str("\n [LLWU ISR] ****WUF15 was set *****\r\n"); 
       LLWU_F2 |= LLWU_F2_WUF15_MASK;   // write one to clear the flag
   }
   if (LLWU_F3 & LLWU_F3_MWUF0_MASK) {
 //      print_str("\n [LLWU ISR] ****WUF3_MWUF0 IF  LPT0  *****\r\n"); 
       LPTMR0_CSR |=  LPTMR_CSR_TCF_MASK;   // write 1 to TCF to clear the LPT timer compare flag
       LPTMR0_CSR = ( LPTMR_CSR_TEN_MASK | LPTMR_CSR_TIE_MASK | LPTMR_CSR_TCF_MASK  );
       LLWU_F3 |= LLWU_F3_MWUF0_MASK;   // write one to clear the flag
   }
   if (LLWU_F3 & LLWU_F3_MWUF1_MASK) {
//	     print_str("\n [LLWU ISR] ****WUF3_MWUF1 IF  LPT1  *****\r\n"); 
	     LPTMR0_CSR |=  LPTMR_CSR_TCF_MASK;   // write 1 to TCF to clear the LPT timer compare flag
	     LPTMR0_CSR = ( LPTMR_CSR_TEN_MASK | LPTMR_CSR_TIE_MASK | LPTMR_CSR_TCF_MASK  );
       LLWU_F3 |= LLWU_F3_MWUF1_MASK;   // write one to clear the flag
   }
   if (LLWU_F3 & LLWU_F3_MWUF2_MASK) {
//	     print_str("\n [LLWU ISR] ****WUF3_MWUF2 IF  CMP0 *****\r\n"); 
       LLWU_F3 |= LLWU_F3_MWUF2_MASK;   // write one to clear the flag
   }
   if (LLWU_F3 & LLWU_F3_MWUF3_MASK) {
//	     print_str("\n [LLWU ISR] ****WUF3_MWUF3 IF  TSI  *****\r\n"); 
       LLWU_F3 |= LLWU_F3_MWUF3_MASK;   // write one to clear the flag
   }
}
