/*
 * File:        sysinit.c
 * Purpose:     Kinetis L Family Configuration
 *              Initializes processor to a default state
 *
 * Notes:
 *
 *
 */

#include "common.h"
#include "sysinit.h"

/********************************************************************/
void sysinit (void)
{
  
        /* Enable all of the port clocks. These have to be enabled to configure
         * pin muxing options, so most code will need all of these on anyway.
         */
/* 
      SIM_SCGC5 |= (SIM_SCGC5_PORTA_MASK
                      | SIM_SCGC5_PORTB_MASK
                      | SIM_SCGC5_PORTC_MASK
                      | SIM_SCGC5_PORTD_MASK
                      | SIM_SCGC5_PORTE_MASK );
*/
        // Release hold with ACKISO:  Only has an effect if recovering from VLLS1, VLLS2, or VLLS3
        // if ACKISO is set you must clear ackiso before calling pll_init 
        //    or pll init hangs waiting for OSC to initialize
        // if osc enabled in low power modes - enable it first before ack
        // if I/O needs to be maintained without glitches enable outputs and modules first before ack.
        if (PMC_REGSC &  PMC_REGSC_ACKISO_MASK)
        PMC_REGSC |= PMC_REGSC_ACKISO_MASK;
        

       /* Ramp up the system clock */
       /* Set the system dividers */
        SIM_CLKDIV1 = ( 0
                        | SIM_CLKDIV1_OUTDIV1(0)
                        | SIM_CLKDIV1_OUTDIV4(1) );
 
}

/*********************************************************************************************/
/* Functon name : pll_init
 *
 * Mode transition: Option to move from FEI to PEE mode or to just initialize the PLL
 *
 * This function initializess PLL0. Either OSC0 is selected for the
 * reference clock source. The oscillators can be configured to use a crystal or take in an
 * external square wave clock.
 * Using the function parameter names the PLL frequency is calculated as follows:
 * PLL freq = ((crystal_val / prdiv_val) * vdiv_val)
 * Refer to the readme file in the mcg driver directory for examples of pll_init configurations.
 * All parameters must be provided, for example crystal_val must be provided even if the
 * oscillator associated with that parameter is already initialized.
 * The various passed parameters are checked to ensure they are within the allowed range. If any
 * of these checks fail the driver will exit and return a fail/error code. An error code will
 * also be returned if any error occurs during the PLL initialization sequence. Refer to the
 * readme file in the mcg driver directory for a list of all these codes.
 *
 * Parameters: crystal_val - external clock frequency in Hz either from a crystal or square
 *                           wave clock source
 *             hgo_val     - selects whether low power or high gain mode is selected
 *                           for the crystal oscillator. This has no meaning if an
 *                           external clock is used.
 *             erefs_val   - selects external clock (=0) or crystal osc (=1)
 *             prdiv_val   - value to divide the external clock source by to create the desired
 *                           PLL reference clock frequency
 *             vdiv_val    - value to multiply the PLL reference clock frequency by
 *             mcgout_select  - 0 if the PLL is just to be enabled, non-zero if the PLL is used
 *                              to provide the MCGOUT clock for the system.
 *
 * Return value : PLL frequency (Hz) or error code
 */

void pll_init(void)
{

  // Setup PLL output 96MHz, Core/system 48MHz, Bus/Flash 24MHz
  SIM_CLKDIV1 = ( 0
                  | SIM_CLKDIV1_OUTDIV1(1)
                  | SIM_CLKDIV1_OUTDIV4(1) );
  
  // configure the MCG_C2 register for high-frequency crystal
  MCG_C2 = (MCG_C2_LOCRE0_MASK | MCG_C2_RANGE0(1) | MCG_C2_EREFS0_MASK);
  
  // Select external oscillator and Reference Divider and clear IREFS to start ext osc
  // CLKS=2, FRDIV=3, IREFS=0, IRCLKEN=0, IREFSTEN=0
  MCG_C1 = (MCG_C1_CLKS(2) | MCG_C1_FRDIV(3)); // Set the required CLKS and FRDIV values

  // need to wait for OSCINIT to set
  for (;;)
  {
    if (MCG_S & MCG_S_OSCINIT0_MASK) break; // jump out OSCINIT sets 
  }

  // wait for Reference clock Status bit to clear
  for (;;)
  {
    if (!(MCG_S & MCG_S_IREFST_MASK)) break; // jump out if IREFST clears 
  }

  // Wait for clock status bits to show clock source is ext ref clk
  for (;;)
  {
    if (((MCG_S & MCG_S_CLKST_MASK) >> MCG_S_CLKST_SHIFT) == 0x2) break; // jump out if CLKST shows EXT CLK slected 
  }

  // Now in FBE
  // It is recommended that the clock monitor is enabled when using an external clock as the clock source/reference.
  // It is enabled here but can be removed if this is not required.
  MCG_C6 |= MCG_C6_CME0_MASK;
  
  // Configure PLL
  // Configure MCG_C5
  MCG_C5 = MCG_C5_PRDIV0(3);    //set PLL ref divider to 4, gives 2MHz

  // Configure MCG_C6
  // The PLLS bit is set to enable the PLL, MCGOUT still sourced from ext ref clk
  // The loss of lock interrupt can be enabled by seperately OR'ing in the LOLIE bit in MCG_C6
  MCG_C6 = MCG_C6_PLLS_MASK | MCG_C6_CME0_MASK | MCG_C6_VDIV0(24); // PLL multiply by 48, gives PLL 96MHz

  // wait for PLLST status bit to set
  for (;;)
  {
    if (MCG_S & MCG_S_PLLST_MASK) break; // jump out if PLLST sets 
  }

  // Wait for LOCK bit to set
  for (;;)
  {
    if (MCG_S & MCG_S_LOCK0_MASK) break; // jump out if LOCK sets 
  }

  // now in PBE

  MCG_C1 &= ~MCG_C1_CLKS_MASK; // clear CLKS to switch CLKS mux to select PLL as MCG_OUT

  // Wait for clock status bits to update
  for (;;)
  {
    if (((MCG_S & MCG_S_CLKST_MASK) >> MCG_S_CLKST_SHIFT) == 0x3) break; // jump out if CLKST = 3 
  }

  // Now in PEE
  SIM_SOPT2 |= SIM_SOPT2_PLLFLLSEL_MASK; // set PLLFLLSEL to select the PLL for this clock source
  
  //clock gate and USB clock source
  SIM_SOPT2 |= SIM_SOPT2_USBSRC_MASK;   // Set USB clock source to PLL/2
  SIM_SCGC4 |= SIM_SCGC4_USBOTG_MASK;   // Enable clock gate for USB
  
  
  return;
} // pll_init
